@@ -1,9 +1,11 @@
 /* common definitions for `patch' */
 
-/* $Id: common.h,v 1.20 1999/08/30 06:20:08 eggert Exp $ */
+/* $Id: common.h,v 1.34 2003/05/19 06:57:36 eggert Exp $ */
 
-/* Copyright 1986, 1988 Larry Wall
-   Copyright 1990, 1991-1993, 1997-1998, 1999 Free Software Foundation, Inc.
+/* Copyright (C) 1986, 1988 Larry Wall
+
+   Copyright (C) 1990, 1991, 1992, 1993, 1997, 1998, 1999, 2002, 2003
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,17 +26,10 @@
 #define DEBUGGING 1
 #endif
 
-/* We must define `volatile' and `const' first (the latter inside config.h),
-   so that they're used consistently in all system includes.  */
-#ifndef __STDC__
-# ifndef volatile
-# define volatile
-# endif
-#endif
-
 #include <config.h>
 
 #include <assert.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <time.h>
@@ -73,6 +68,9 @@
 #ifndef S_IRUSR
 #define S_IRUSR (S_IROTH << 6)
 #endif
+#ifdef MKDIR_TAKES_ONE_ARG
+# define mkdir(name, mode) ((mkdir) (name))
+#endif
 
 #if HAVE_LIMITS_H
 # include <limits.h>
@@ -123,27 +121,6 @@
 #endif
 
 
-#ifndef FILESYSTEM_PREFIX_LEN
-#define FILESYSTEM_PREFIX_LEN(f) 0
-#endif
-
-#ifndef ISSLASH
-#define ISSLASH(c) ((c) == '/')
-#endif
-
-
-/* constants */
-
-/* AIX predefines these.  */
-#ifdef TRUE
-#undef TRUE
-#endif
-#ifdef FALSE
-#undef FALSE
-#endif
-#define TRUE 1
-#define FALSE 0
-
 /* handy definitions */
 
 #define strEQ(s1,s2) (!strcmp(s1, s2))
@@ -151,12 +128,11 @@
 
 /* typedefs */
 
-typedef int bool;			/* must promote to itself */
 typedef off_t LINENUM;			/* must be signed */
 
 /* globals */
 
-extern char const program_name[];
+XTERN char *program_name;	/* The name this program was run with. */
 
 XTERN char *buf;			/* general purpose buffer */
 XTERN size_t bufsize;			/* allocated size of buf */
@@ -190,14 +166,14 @@ XTERN int debug;
 XTERN bool force;
 XTERN bool batch;
 XTERN bool noreverse;
-XTERN int reverse;
+XTERN bool reverse;
 XTERN enum { DEFAULT_VERBOSITY, SILENT, VERBOSE } verbosity;
 XTERN bool skip_rest_of_patch;
 XTERN int strippath;
 XTERN bool canonicalize;
 XTERN int patch_get;
-XTERN int set_time;
-XTERN int set_utc;
+XTERN bool set_time;
+XTERN bool set_utc;
 
 enum diff
   {
@@ -213,25 +189,11 @@ XTERN enum diff diff_type;
 
 XTERN char *revision;			/* prerequisite revision, if any */
 
-#ifdef __STDC__
-# define GENERIC_OBJECT void
-#else
-# define GENERIC_OBJECT char
-#endif
-
 #if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 6) || __STRICT_ANSI__
 # define __attribute__(x)
 #endif
 
-#ifndef PARAMS
-# ifdef __STDC__
-#  define PARAMS(args) args
-# else
-#  define PARAMS(args) ()
-# endif
-#endif
-
-void fatal_exit PARAMS ((int)) __attribute__ ((noreturn));
+void fatal_exit (int) __attribute__ ((noreturn));
 
 #include <errno.h>
 #if !STDC_HEADERS && !defined errno
@@ -244,7 +206,7 @@ extern int errno;
 # if !HAVE_MEMCHR
 #  define memcmp(s1, s2, n) bcmp (s1, s2, n)
 #  define memcpy(d, s, n) bcopy (s, d, n)
-GENERIC_OBJECT *memchr ();
+void *memchr ();
 # endif
 #endif
 
@@ -252,8 +214,8 @@ GENERIC_OBJECT *memchr ();
 # include <stdlib.h>
 #else
 char *getenv ();
-GENERIC_OBJECT *malloc ();
-GENERIC_OBJECT *realloc ();
+void *malloc ();
+void *realloc ();
 #endif
 
 #if HAVE_UNISTD_H
@@ -284,6 +246,13 @@ GENERIC_OBJECT *realloc ();
 # define file_seek fseek
 # define file_tell ftell
 #endif
+#if ! (HAVE_GETEUID || defined geteuid)
+# if ! (HAVE_GETUID || defined getuid)
+#  define geteuid() (-1)
+# else
+#  define geteuid() getuid ()
+# endif
+#endif
 
 #if HAVE_FCNTL_H
 # include <fcntl.h>
@@ -313,7 +282,7 @@ GENERIC_OBJECT *realloc ();
 #define O_TRUNC 0
 #endif
 
-#if HAVE_SETMODE
+#if HAVE_SETMODE_DOS
   XTERN int binary_transput;	/* O_BINARY if binary i/o is desired */
 #else
 # define binary_transput 0
